/*!
  \file   apriori/AprioriSelector.hpp
  \brief  This class processes some program argument of main-fsm.cpp, 
  initializes some classes, and selects the Apriori algorithm to run.


  \author Ferenc BODON
  \date   2005-05-02
*/

#ifndef AprioriSelector_HPP
#define AprioriSelector_HPP

#include "common/allocators.hpp"


#include "apriori/bodon/dynamic_trie/trie_manipulators/FrequentItemInserter.hpp"
#include "apriori/bodon/dynamic_trie/trie_manipulators/FrequentPairInserter.hpp"
#include "apriori/bodon/dynamic_trie/trie_manipulators/FrequentPairInserterNoprune.hpp"

#include "apriori/bodon/dynamic_trie/trie_manipulators/IntersectProPruner.hpp"
#include "apriori/bodon/dynamic_trie/trie_manipulators/CandidateGeneratorPrune.hpp"
#include "apriori/bodon/dynamic_trie/trie_manipulators/CandGenInfreqRemoveNopruneMerge.hpp"
#include "apriori/bodon/dynamic_trie/trie_manipulators/InfreqRemover.hpp"

#include "apriori/Apriori.hpp"

/**
   This class processes some program argument of main-fsm.cpp, 
   initializes some classes, and selects the Apriori algorithm to run.

   \param MAIN_DATA_STRUCTURE the main data structure. It is basically a trie (prefix-trie)
   \param SEC_DATA_STRUCTURE the secondary data structure
   \param LEAF_WC an other data structure
   \param CODER a coder class, which is most of the cases Coder
   \param F2F a frequent pair filter
   \param SUPP_C a support counter class, it has to implement interface SupportCounterInterface
   \param T_R a transaction reader class, it has to implement interface TransactionReader
   \param DF_D type of the depth-first decoder
   \param NEE the level of equisupport pruning

 */
template <class MAIN_DATA_STRUCTURE, class SEC_DATA_STRUCTURE, class LEAF_WC, 
	  class CODER, class F2F, class SUPP_C, class T_R, 
	  class DF_D, NEELevel NEE = NEE_Full> class AprioriSelector
{
   public:
      AprioriSelector( 
	 counter_t min_supp, char* algorithm, char* input_file,
	 counter_t nr_of_transactions, 
	 std::vector< std::pair<counter_t, item_t> >& freq_items_with_counters, 
	 T_R& tr_reader, DF_D& df_decoder, unsigned int maxsize = largest_itemsetsize);
};

template <class MAIN_DATA_STRUCTURE, class SEC_DATA_STRUCTURE, class LEAF_WC, 
	  class CODER, class F2F, class SUPP_C, class T_R, class DF_D, 
	  NEELevel NEE>
AprioriSelector<MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, CODER, F2F, 
		SUPP_C, T_R, DF_D, NEE>::AprioriSelector( 
   counter_t min_supp, char* algorithm, char* input_file,
   counter_t nr_of_transactions, 
   std::vector< std::pair<counter_t, item_t> >& freq_items_with_counters, 
   T_R& tr_reader, DF_D& df_decoder, unsigned int maxsize )
{
   typename CODER::params_t par_c;
   par_c.file_name = input_file;
   par_c.mode=FileReprBase::READ;
   par_c.largest_item = tr_reader.getLargestItem();
   par_c.decoder = &df_decoder;
   par_c.freq_items_with_counters = &freq_items_with_counters;
   par_c.codemode = ASC;
//   par_c.codemode = DESC;
   log_status(0,"Doing sorted codec.");
   CODER sorted_coder(&par_c);

   std::vector< std::pair<counter_t, std::pair<item_t, item_t> > >
      freq_pairs_with_counters;
   F2F fr_2_filter(&sorted_coder );
   log_status(0,"Finding frequent pairs.")
      fr_2_filter.findFrequentPairs(freq_pairs_with_counters, min_supp);

   MAIN_DATA_STRUCTURE main_trie;
   typedef bracz::singleualloc<LEAF_WC, 64 * 1024> LEAF_ALLOCATOR;
   LEAF_ALLOCATOR s_alloc;
   typedef Bodon::dynamic_trie::FrequentItemInserter<DF_D, MAIN_DATA_STRUCTURE, NEE> FII;
   FII fii(main_trie, df_decoder);
   char *kind=algorithm+6;


   if( strstr(kind,"-noprune") )
   {
      log_status(0,"Complete pruning is disabled.");
      typedef Bodon::dynamic_trie::FrequentPairInserterNoprune<
	 DF_D, MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, LEAF_ALLOCATOR, NEE> FPI;
      typedef Bodon::dynamic_trie::CandGenInfreqRemoveNopruneMerge<
	 DF_D, MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, LEAF_ALLOCATOR, NEE > CGIR;
      CGIR infrequent_remover(main_trie, df_decoder, s_alloc);

      typedef Apriori<CODER, DF_D, MAIN_DATA_STRUCTURE, LEAF_ALLOCATOR, FII, 
	 FPI, CGIR, CGIR, SUPP_C> A;
      A apriori(main_trie, s_alloc, infrequent_remover, sorted_coder, df_decoder, fii);
      log_status(0,"Finding frequent itemsets.")
	 apriori.findFrequentItemsets( 
	    nr_of_transactions, *par_c.freq_counters,
	    freq_pairs_with_counters, min_supp, maxsize );
   }
   else
   {
      log_status(0,"Intersection-based pruning is selected.");
      typedef Bodon::dynamic_trie::FrequentPairInserter<DF_D, MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, 
	 LEAF_ALLOCATOR, NEE> FPI;
      typedef Bodon::dynamic_trie::IntersectProPruner<
	 DF_D, MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, LEAF_ALLOCATOR, NEE> PRUNER;
      typedef Bodon::dynamic_trie::CandidateGeneratorPrune<PRUNER, DF_D, 
	 MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, LEAF_ALLOCATOR, NEE> CG;
      typedef Bodon::dynamic_trie::InfreqRemover<
	 DF_D, MAIN_DATA_STRUCTURE, SEC_DATA_STRUCTURE, LEAF_WC, LEAF_ALLOCATOR, NEE> IR;
      IR infrequent_remover(main_trie, df_decoder, s_alloc);
      typedef Apriori<CODER, DF_D, MAIN_DATA_STRUCTURE, LEAF_ALLOCATOR, FII, FPI, CG, IR, SUPP_C> A;
      A apriori(main_trie, s_alloc, infrequent_remover, sorted_coder, df_decoder, fii);
      log_status(0,"Finding frequent itemsets.");
      apriori.findFrequentItemsets( 
	 nr_of_transactions, *par_c.freq_counters,
	 freq_pairs_with_counters, min_supp, maxsize );
   }
}

#endif
